# encoding: utf-8

"""
Exposes a config_load_test() function and TestLoader subclass.

"""

import unittest

def _test_gen(tests):
    """
    Return a generator over all TestCase instances recursively in tests.

    Sample usage--

    for test in _test_gen(tests):
        print test

    The point is that TestSuite.__iter__() only provides direct access to
    child tests and not to grandchild tests, etc. which this function
    provides.  Also see--

      http://docs.python.org/library/unittest.html#unittest.TestSuite.__iter__

    Arguments:

      tests: a TestCase instance, TestSuite instance, or iterable of
        TestCase and TestSuite instances.

    """
    if isinstance(tests, unittest.TestCase):
        yield tests
        return
    # Otherwise, we have an iterable or a TestSuite instance.
    for test in tests:
        for test2 in _test_gen(test):
            yield test2

def config_load_tests(loader, tests, pattern):
    """
    A unittest load_tests protocol implementation and helper.

    This function adds a test_config attribute to every test case with value
    loader.test_config and returns a unittest.TestSuite instance.

    This function lets one pass configuration data to TestCase classes without
    relying on global variables.  The configuration data can be accessed
    from within TestCase objects as `self.test_config`.

    To use this API, the unittest.TestLoader object that you pass to your
    unittest.TestProgram class must have a test_config attribute set to the
    object with which you would like to store configuration data.

    Usage
    -----

    First store configuration data in your test loader:

        loader = TestLoader()  # can be unittest.TestLoader
        loader.test_config = config_data
        unittest.main(testLoader=loader)

    Then from within the test modules that need it:

        from pizza.test.harness.general import config_load_tests

        load_tests = config_load_tests  # trigger the load_tests protocol.

    You can also use config_load_tests() in addition to an existing
    load_tests() implementation.  For example:

        def load_tests(loader, tests, pattern):
            # Do stuff: make and/or modify your tests, etc.
            return config_load_tests(loader, tests, pattern)

    """
    for test in _test_gen(tests):
        test.test_config = loader.test_config

    return unittest.TestSuite(tests)

class TestLoader(unittest.TestLoader):

    """
    This TestLoader differs from unittest's default TestLoader by providing
    additional diagnostic information when an AttributeError occurs while
    loading a module.

    Because of Python issue 7559 ( http://bugs.python.org/issue7559# ),
    the unittest module masks ImportErrors and the name of the offending
    module.  This TestLoader reports the name of the offending module
    along with a reminder that the AttributeError may be masking an
    ImportError.

    """

    def loadTestsFromNames(self, names, module=None):
        """
        Return a suite of all unit tests and doctests in the package.

        """
        suites = []

        for name in names:
            try:
                suite = self.loadTestsFromName(name, module)
            except AttributeError as err:
                msg = """AttributeError while loading unit tests from:
%s

Due to a bug in Python's unittest module, the AttributeError may be masking
an ImportError in the module being processed.
-->%s""" % (repr(name), str(err))
                # TODO [template]: add to the existing exception instead of
                # raising a new one.
                raise Exception(msg)
            suites.append(suite)

        return self.suiteClass(suites)



"""
Provides the function for the main setup.py console_script.

"""

import logging
import os
import sys
import traceback

import pizza.pizza as _pizza
import pizza.general.common as _common
import pizza.general.logconfig as logconfig
import pizza.general.optionparser as _parsing
import pizza.scripts
import pizza.scripts.pizza.argparsing as argparsing
import pizza.test.harness.main as harness

EXIT_STATUS_SUCCESS = 0
EXIT_STATUS_FAIL = 1
EXIT_STATUS_USAGE_ERROR = 2

LOGGING_LEVEL_DEFAULT = logging.INFO

# TODO [template]: should this be made public with a better name?
log = logging.getLogger("pizza.script")
# Loggers that should display during testing.
# TODO [template]: make this test_loggers instead of test_logger_names.
test_logger_names = [logger.name for logger in (log, harness.log)]


def error(msg, add_trace=False):
    if add_trace:
        msg = traceback.format_exc()
    log.error(msg)


# TODO [template]: make this testable.
# TODO [template]: finish documenting this method.
# TODO [template]: improve parameter names.
def _configure_logging(level=None, stream=None, is_testing=False,
                       is_verbose=False):
    """
    Arguments:

      level: lowest logging level to log.
      stream: the stream to which to log (e.g. sys.stderr).

    """
    if stream is None:
        stream = sys.stderr

    level = logging.DEBUG if is_verbose else LOGGING_LEVEL_DEFAULT

    # We pass a newline as last_text to prevent a newline from being added
    # before the first log message.
    rstream = logconfig.RememberingStream(stream, last_text='\n')
    handler = logconfig.NewlineStreamHandler(rstream)

    if is_testing:
        # Then only log designated test loggers.
        class Filter(object):
            def filter(self, record):
                return record.name in test_logger_names
        handler.addFilter(Filter())
    if not is_verbose:
        # Then shorten the logger name if long.
        class Filter(object):
            def filter(self, record):
                """Set record.truncated_name."""
                parts = record.name.split(".")
                if len(parts) <= 3:
                    truncated_name = record.name
                else:
                    truncated_name = '.'.join(parts[:2] + ['.', parts[-1]])
                record.truncated_name = truncated_name
                return True
        handler.addFilter(Filter())

    # Prefix log messages unobtrusively with "log" to distinguish log
    # messages more obviously from other text sent to the error stream.
    format_string = ("log: %%(%s)s: [%%(levelname)s] %%(message)s" %
                     ('name' if is_verbose else 'truncated_name'))

    formatter = logging.Formatter(format_string)
    handler.setFormatter(formatter)

    root = logging.getLogger()
    root.setLevel(level)
    # Adding at least one handler to the root logger prevents the following
    # message from ever being logged:
    # "No handlers could be found for logger..."
    root.addHandler(handler)

    log.debug("debug logging enabled")


def configure_logging(argv, stream=None):
    """
    Configure logging and return whether to run in verbose mode.

    """
    if argv is None:
        argv = sys.argv
    if stream is None:
        stream = sys.stderr

    is_testing = False
    is_verbose = False

    # Configure logging before parsing arguments for real.
    ns = argparsing.preparse_args(argv)

    if ns is not None:
        # Then args parsed without error.
        is_verbose = ns.verbose
        if ns.run_tests:
            is_testing = True

    # TODO [template]: reconsider the argument names here.
    _configure_logging(stream=stream, is_testing=is_testing,
                       is_verbose=is_verbose)

    return is_verbose


def _main_inner(argv, from_source):
    """Run the program and return the status code."""
    argv = list(argv)  # since we'll be modifying this.

    pizza_dir = os.path.dirname(pizza.__file__)

    if from_source:
        # TODO [template]: expose this path calculation in a more central module.
        sdist_dir = os.path.join(pizza_dir, os.pardir)
        argv[1:1] = ['--sdist-dir', sdist_dir]
        start_dir = sdist_dir
    else:
        start_dir = pizza_dir

    log.debug("argv: %r" % argv)
    ns = argparsing.parse_args(argv)
    log.debug("parsed args: %r" % ns)
    log.debug("cwd: %r" % os.getcwd())

    if ns.run_tests is not None:  # Then the value is a list.
        test_argv = ([argv[0], 'discover', '--start-directory', start_dir] +
                     ns.run_tests)
        harness.run_tests(test_argv)
    else:
        values = ns.args
        result = _pizza.run_pizza(values)
        print(result)


def _main(argv=None, from_source=False):
    if argv is None:
        argv = sys.argv

    verbose = configure_logging(argv, stream=sys.stderr)
    # TODO [template]: also handle KeyboardInterrupt?
    try:
        _main_inner(argv, from_source)
        status = EXIT_STATUS_SUCCESS
    except _parsing.UsageError as err:
        details = """\
Usage error: %s
-->argv: %r
Pass %s for help documentation and available options.""" % (
            err, sys.argv, argparsing.OPTION_HELP.display(' or '))
        error(details, verbose)
        status = EXIT_STATUS_USAGE_ERROR
    except _common.Error, err:
        msg = """\
%s
Pass %s for the stack trace.""" % (err,
                                   argparsing.OPTION_VERBOSE.display(' or '))
        error(msg, add_trace=verbose)
        status = EXIT_STATUS_FAIL
    except Exception, err:
        # Always add the stack trace for "unexpected" exceptions.
        error(err, add_trace=True)
        status = EXIT_STATUS_FAIL

    return status


# We follow most of Guido van Rossum's 2003 advice regarding main()
# functions (though we choose _main() as the function that returns an exit
# status rather than main()):
# http://www.artima.com/weblogs/viewpost.jsp?thread=4829
def main(argv=None, from_source=False):
    """
    Arguments:

      from_source: whether this function is being called from a source
        checkout (e.g. by running `python runpizza.py` or
        `python -m pizza.scripts.pizza`).

    """
    status = _main(argv, from_source=from_source)
    sys.exit(status)




import unittest

# TODO [template]: code this stub class.
class GetDiffPathsTestCase(unittest.TestCase):

    # TODO [template]: add a test case for excluding an item in the project
    # directory.
    def test(self):
        self.assertEquals(1, 1)


if __name__ == '__main__':
    unittest.main()



# encoding: utf-8
#
# Copyright (C) 2011-2013 Chris Jerdonek. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * The names of the copyright holders may not be used to endorse or promote
#   products derived from this software without specific prior written
#   permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

"""
Supports command-line parsing.

"""

import argparse


class UsageError(Exception):
    """
    Exception class for command-line syntax errors.

    """
    pass


class Option(tuple):
    """
    Encapsulates a command option (e.g. "-h" and "--help", or "--run-tests").

    """
    def display(self, glue):
        return glue.join(self)


class ArgParser(argparse.ArgumentParser):

    def error(self, message):
        """
        Handle an error occurring while parsing command arguments.

        This method overrides the base class's error(), which prints a
        usage message to standard error and terminates the program with
        a status code of 2.

        """
        raise UsageError(message)



# encoding: utf-8

"""
Utility functions for setup.py.

"""

import filecmp
import logging
import os

ENCODING_DEFAULT = 'utf-8'

_log = logging.getLogger(os.path.basename(__name__))


def read(path, encoding=None):
    """
    Read and return the contents of a text file as a unicode string.

    """
    if encoding is None:
        encoding = ENCODING_DEFAULT

    # This implementation was chosen to be compatible across Python 2/3.
    with open(path, 'rb') as f:
        b = f.read()

    return b.decode(encoding)


def _write(b, path):
    """
    Write bytes to a path.

    """
    with open(path, 'wb') as f:
        f.write(b)

def write(u, path, encoding=None, description=None):
    """
    Write a unicode string to a file.

    """
    if encoding is None:
        encoding = ENCODING_DEFAULT

    desc = ('%s ' % description) if description else ''
    _log.info("writing %sto: %s" % (desc, path))
    # This implementation was chosen to be compatible across Python 2/3.
    b = u.encode(encoding)
    try:
        _write(b, path)
    except IOError:
        # Retry if directory wasn't there a la EAFP.
        dirname = os.path.dirname(path)
        if not os.path.exists(dirname):
            _log.info("creating directory: %s" % dirname)
            os.makedirs(dirname)
            _write(b, path)
        else:
            raise

# In setup.py, scraping the version number from the package is preferable to
# importing the package because setup.py should not have to be able to
# run the package in order to do its distribution-related tasks.  Also,
# importing might not always be possible anyways, for example if the package
# is written in Python 2 but setup.py is being run from Python 3.  See this
# e-mail exchange for a brief discussion, for example:
# http://mail.python.org/pipermail/python-porting/2012-May/000298.html
def scrape_version(package_dir):
    """
    Return the version string for a package without importing the package.

    For this function to work, the package's __init__.py file must contain
    a line of the following form:

        __version__ = '0.1.2-alpha'

    """
    init_path = os.path.join(package_dir, '__init__.py')
    text = read(init_path)

    needle = '__version__ ='
    try:
        line = next(line for line in text.splitlines()
                    if line.startswith(needle))
    except StopIteration:
        raise Exception("version string not found in: %s" % init_path)

    expr = line[len(needle):]
    # Using eval() is more robust than using a regular expression.
    # For example, this method can handle single and double quotes,
    # end-of-line comments, and more complex version string expressions.
    version = eval(expr)

    return version


class Results(object):

    attrs = ('project_only', 'sdist_only', 'not_skipped', 'sdist_funny')

    def __init__(self):
        for attr in self.attrs:
            setattr(self, attr, [])

class Differencer(object):

    """Comparse project directories to generated sdist directories."""

    def __init__(self, skip):
        self.skip = skip
        # Each element is a: (dircmp attribute name, filter function)
        # These yield the paths added to the Results object in each step of
        # the recursive comparison.
        self.dcmp_filters = (('left_only', lambda path: not skip(path)),
                             ('right_only', lambda path: True),
                             ('common_files', lambda path: skip(path)),
                             ('common_funny', lambda path: True))

    def _get_diff_paths(self, dcmp, results, rel_parent_dir=''):
        """
        Recursively compare the given directory pair.

        This method modifies the results object in place instead of returning
        a value.

        Arguments:

          dcmp: a filecmp.dircmp object

          results: a Results object.

          rel_parent_dir: the directory relative to the project/sdist root.

          skip: a function that accepts a path relative to the project
            directory and returns whether that file should be skipped instead
            of copied.

        """
        new_rel = lambda name: os.path.join(rel_parent_dir, name)

        for rattr, (dattr, filt) in zip(Results.attrs, self.dcmp_filters):
            result_paths = getattr(results, rattr)
            dcmp_paths = [new_rel(name) for name in getattr(dcmp, dattr)]
            result_paths.extend([path for path in dcmp_paths if filt(path)])

        not_skipped = results.not_skipped
        for name in dcmp.common_dirs:
            rel_path = new_rel(name)
            if self.skip(rel_path):
                not_skipped.append(rel_path)
            self._get_diff_paths(dcmp.subdirs[name], results,
                                 rel_parent_dir=rel_path)

    def get_results(self, project_dir, sdist_dir):
        """
        Return differences between directories as a Results object.

        """
        dcmp = filecmp.dircmp(project_dir, sdist_dir)
        results = Results()
        self._get_diff_paths(dcmp, results)
        return results


# TODO [template]: consider moving this formatting function to setup.py.
def describe_differences(project_dir, sdist_dir, skip=None, indent='  '):
    """
    Describe the differences between the project and sdist directories.

    Returns a string.

    Arguments:

      skip: a function that accepts a path relative to the project directory
        and returns whether that file should be skipped instead of copied.

    """
    if skip is None:
        skip = lambda path: False

    differencer = Differencer(skip)
    results = differencer.get_results(project_dir, sdist_dir)

    def format(header, paths):
        header = '%s:' % header
        if not paths:
            header += " none"
        strings = [header] + sorted(paths)
        glue = '\n' + indent
        return glue.join(strings)

    headers = ('Only in project', 'Only in sdist', 'Not skipped', 'Funny in sdist')

    sections = [format(header, getattr(results, attr))
                for header, attr in zip(headers, Results.attrs)]
    return "\n".join(sections)


def strip_html_comments(source_path):
    """
    Read the file, and strip HTML comments.

    Returns a unicode string.

    """
    # This function assumes no line mixes HTML comments and non-comments.
    text = read(source_path)
    lines = text.splitlines(True)  # preserve line endings.

    new_lines = []
    exclude = False
    for line in lines:
        if line.startswith("<!--"):
            exclude = True
        if not exclude:
            new_lines.append(line)
        if line.find("-->") >= 0:
            exclude = False

    return "".join(new_lines)


def make_temp_path(path, temp_dir, label=None, new_ext=None):
    """
    Arguments:

      new_ext: the new file extension, including the leading dot.
        Defaults to preserving the file extension.

    """
    if label is None:
        label = ''
    basename = os.path.basename(path)
    name, ext = os.path.splitext(basename)
    if new_ext is None:
        new_ext = ext
    name = "%s%s%s" % (name, label, new_ext)
    return os.path.join(temp_dir, name)


def _write_md_to_rst(source_path, target_path, docstring_path):
    """
    Convert the given file from markdown to reStructuredText.

    Returns the path to a UTF-8 encoded file.

    """
    # Pandoc uses UTF-8 for both input and output.
    command = "pandoc --write=rst --output=%s %s" % (target_path, source_path)
    _log.info("converting with pandoc: %s to %s\n-->%s" %
              (source_path, target_path, command))

    if os.path.exists(target_path):
        os.remove(target_path)

    os.system(command)

    if not os.path.exists(target_path):
        _log.error("Error running: %s\n"
                   "  Did you install pandoc per the %s docstring?" %
                   (command, docstring_path))
        sys.exit(1)

    return target_path


def convert_md_to_rst(source_path, temp_rst_path, docstring_path):
    """
    Convert the file contents from markdown to reStructuredText.

    Returns the converted contents as a unicode string.

    Arguments:

      source_path: the path to the UTF-8 encoded file to be converted.

      docstring_path: the path to the Python file whose docstring contains
        instructions on how to install pandoc.

    """
    _write_md_to_rst(source_path, temp_rst_path, __file__)
    return read(temp_rst_path, encoding='utf-8')


def update_description_file(source_paths, target_path, docstring_path,
                            temp_dir='temp'):
    """
    Write the long_description needed for setup.py to a file.

    The long description needs to be formatted as reStructuredText:

      http://docs.python.org/distutils/setupscript.html#additional-meta-data

    """
    # Remove our HTML comments because PyPI does not allow it.
    # See the setup.py docstring for more info on this.
    sections = [strip_html_comments(path) for path in source_paths]
    md_description = '\n\n'.join(sections)

    md_ext = os.path.splitext(source_paths[0])[1]  # e.g. '.md'
    temp_md_path = make_temp_path(target_path, temp_dir=temp_dir,
                                  label=".prelim", new_ext=md_ext)
    write(md_description, temp_md_path, encoding='utf-8',
          description='preliminary long_description')
    temp_rst_path = make_temp_path(target_path, temp_dir=temp_dir)
    rst_description = convert_md_to_rst(source_path=temp_md_path,
                                        temp_rst_path=temp_rst_path,
                                        docstring_path=docstring_path)

    # Comments in reST begin with two dots.
    intro_text = """\
.. This file is auto-generated by setup.py for PyPI using pandoc, so this
.. file should not be edited.  Edits should go into the files from which
.. this file is constructed.
..
.. This file contains the long_description argument to setup.py's setup().
.. It should be checked into source control and be part of the source
.. distribution so that setup() can be passed the long_description argument
.. by end-users even without pandoc (e.g. non-maintainers, etc).

"""

    rst_description = '\n'.join([intro_text, rst_description])


    write(rst_description, target_path,
          description='complete long_description')



# encoding: utf-8
#
# Copyright (C) 2011-2012 Chris Jerdonek. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * The names of the copyright holders may not be used to endorse or promote
#   products derived from this software without specific prior written
#   permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

"""
Supports logging configuration.

"""

import logging


class RememberingStream(object):

    """
    A stream that "remembers" the last text sent to write().

    """

    def __init__(self, stream, last_text=''):
        self._stream = stream
        self._last_text = last_text

    def last_char(self):
        if self._last_text:
            return self._last_text[-1]

    def write(self, text):
        if not text:
            return
        self._stream.write(text)
        self._last_text = text

    # A flush() method is needed to be able to pass instances of this
    # class to unittest.TextTestRunner's constructor.
    def flush(self):
        self._stream.flush()

class NewlineStreamHandler(logging.StreamHandler):

    """
    A log handler whose log messages always start at the beginning of a line.

    This class is useful for preventing output like the following during
    unittest runs:

    ........................log: [INFO] foo
    ..................................................

    The stream attribute (i.e. the stream passed to this class's
    constructor) must implement stream.last_char().

    See also CPython issue #16889: http://bugs.python.org/issue16889
    """

    def emit(self, record):
        if self.stream.last_char() != "\n":
            self.stream.write("\n")

        super(NewlineStreamHandler, self).emit(record)



# encoding: utf-8
# Project auto-generated at: ...
# Copyright (C) 2012 Mustachioed Maven.


# TODO: replace this placeholder docstring and function body with the
# real implementation.
def run_pizza(values):
    """
    Return the number of values.

    This function executes the Pizza package's main command-line functionality.

    """
    return len(values)



#!/usr/bin/env python
# coding: utf-8

"""
Standard Python setup script to support distribution-related tasks.

For instructions on releasing Pizza and on how to use this script, consult
the `releasing.md` file in the `docs` folder of a source distribution.  For
installation and usage instructions, consult the README or the project page:

https://github.com/cjerdonek/groome-python-expected

"""

DISTUTILS_DEBUG = False
# This is a hack to make it easier to enable distutils's debug mode.  This
# code must come before importing from distutils.  See also:
# http://docs.python.org/2/distutils/setupscript.html#debugging-the-setup-script
import os
os.environ['DISTUTILS_DEBUG'] = "1" if DISTUTILS_DEBUG else ""

# We use setuptools/Distribute because distutils does not seem to support
# the following arguments to setUp().  Passing these arguments to
# setUp() causes a UserWarning to be displayed.
#
#  * entry_points
#  * install_requires
#

import distutils
# Distribute/setuptools does not expose all distutils classes.
from distutils.cmd import Command
# We use distutils's upload command because Distribute's seems not to
# work as well in certain circumstances.  For example, see the following
# Distribute bug reports:
#
#   https://bitbucket.org/tarek/distribute/issue/346
#   https://bitbucket.org/tarek/distribute/issue/348
#
import distutils.command.upload as upload_mod
import filecmp
import fnmatch
import logging
import os
import sys

# For various reasons, we deliberately avoid importing from the pizza package
# itself and making the pizza setup code dependent on pizza.  However, it
# is of course okay to import from pizza_setup.
import pizza_setup.utils as utils

# TODO [template]: explore whether I can support distutils (at least for installers).
#
# Whether to use Distribute (or setuptools if not available) over distutils.
# This boolean is temporary for more convenient testing/experimentation.
USE_SETUPTOOLS = True
dist_version = None

if USE_SETUPTOOLS:
    # Distribute does not seem to support the -r/--repository option
    # with the register command (at least without a [server-login] section
    # in the .pypirc).  See Distribute issue #346 :
    # https://bitbucket.org/tarek/distribute/issue/346/upload-fails-without-server-login-but
    import setuptools
    import setuptools.command.register as register_mod
    import setuptools.command.sdist as sdist_mod
    import pkg_resources  # included with Distribute.
    try:
        # This is different from setuptools.__version__, which is always '0.6'
        # in the case of Distribute.
        dist_version = pkg_resources.get_distribution("distribute").version
    except pkg_resources.DistributionNotFound:
        # This is needed to support setuptools if Distribute is not available.
        dist_version = pkg_resources.get_distribution("setuptools").version
    setup = setuptools.setup
    dist = setuptools
else:
    import distutils.command.register as register_mod
    import distutils.command.sdist as sdist_mod
    import distutils.core
    setup = distutils.core.setup
    dist = distutils

PACKAGE_NAME = 'pizza'

README_PATH = 'README.md'
HISTORY_PATH = 'HISTORY.md'
TEMP_DIR = 'temp'
LONG_DESCRIPTION_PATH = 'setup_long_description.rst'
# Top-level names to skip when displaying sdist differences.
SDIST_NAMES_TO_SKIP = ('.git', '.tox', 'build', 'dist', 'temp')

_log = logging.getLogger(os.path.basename(__file__))


def configure_logging():
    """
    Configure setup.py logging with simple settings.

    """
    # Prefix the log messages to distinguish them from other text sent to
    # the error stream.
    format_string = ("%s: %%(name)s: [%%(levelname)s] %%(message)s" %
                     PACKAGE_NAME)
    logging.basicConfig(format=format_string, level=logging.INFO)
    _log.debug("Debug logging enabled.")


def error(msg):
    """
    Exit with an error message

    """
    _log.error(msg)
    sys.exit(1)


def get_long_description():
    path = LONG_DESCRIPTION_PATH
    try:
        long_description = utils.read(path)
    except IOError:
        if not os.path.exists(path):
            raise Exception("long_description file missing: %s" % path)
        raise
    return long_description


def write_long_description(target_path):
    utils.update_description_file([README_PATH, HISTORY_PATH],
                                  target_path,
                                  docstring_path=__file__)

def check_long_description():
    """
    Check whether the prep command needs to be run.

    """
    description_path = LONG_DESCRIPTION_PATH
    temp_path = utils.make_temp_path(description_path, temp_dir=TEMP_DIR)
    write_long_description(temp_path)

    if not filecmp.cmp(temp_path, description_path, shallow=False):
        error("""\
long_description out of date: %s
To update, run the following command and commit the changes--

    python setup.py %s
""" % (description_path, pizza_prep.__name__))

    print("long_description is current: %s" % description_path)


class Reporter(object):

    def __init__(self, project_dir, sdist_dir):
        self.project_dir = project_dir
        self.sdist_dir = sdist_dir

    def make_report(self):
        """
        Display how the sdist differs from the project directory.

        """
        def skip(path):
            dir_path, base_name = os.path.split(path)
            if (base_name.endswith('.pyc') or
                base_name in ('__pycache__', '.DS_Store')):
                return True
            # distutils puts the temporary sdist directory (the directory from
            # which it builds the compressed sdist) into the source tree.
            if path in SDIST_NAMES_TO_SKIP or path == self.sdist_dir:
                return True
            return False

        return utils.describe_differences(self.project_dir, self.sdist_dir,
                                          skip=skip)

    def show_report(self, report, archives, egg_dirs):
        display = """\

Listing unexpected differences between directories--

  project: %s
  sdist: %s (now in %s)

%s
""" % (self.project_dir, self.sdist_dir, archives, report)

        if egg_dirs:
            # For more information on this, see:
            # https://bitbucket.org/tarek/distribute/issue/350
            display += """\

Note: Since an egg-info directory was present when running this command,
the sdist may not reflect recent changes to MANIFEST.in.  Delete the
following egg-info directory and rerun for an up-to-date listing:
%r""" % egg_dirs
        print(display)


# New and customized setup() commands
#
# For commands that write to PyPI, we add a user prompt displaying the URL
# to reduce the chance of accidentally writing to the real PyPI.

class CommandMixin(object):
    def confirm_repository(self):
        command_name = self.get_command_name()
        # The repository attribute is the URL.
        answer = raw_input("Are you sure you want to %s to %s (yes/no)? " %
                           (command_name, self.repository))
        if answer != "yes":
            error("aborted command: %s" % command_name)

class upload(upload_mod.upload, CommandMixin):
    def run(self):
        check_long_description()
        self.confirm_repository()
        return upload_mod.upload.run(self)

class register(register_mod.register, CommandMixin):
    # We override post_to_server() instead of run() because finalize_options()
    # and self._set_config() are called at the beginning of run().
    def post_to_server(self, data, auth=None):
        check_long_description()
        self.confirm_repository()
        return register_mod.register.post_to_server(self, data, auth=auth)

# This command differs from the original by displaying a report showing
# differences between the project repository and the source distribution.
# This is useful in double-checking MANIFEST.in.
class sdist(sdist_mod.sdist):
    def run(self):
        # Check for the presence of a project "egg-info" directory since
        # this can mask what the current MANIFEST.in yields.
        egg_dirs = fnmatch.filter(os.listdir(os.curdir), '*.egg-info')

        # We repeat some of distutils's sdist.make_distribution() logic here.
        _saved_keep_temp = self.keep_temp
        # Tell make_distribution() not to delete the release tree from
        # which it will create the zipped sdist.
        self.keep_temp = True
        sdist_mod.sdist.run(self)
        base_dir = self.distribution.get_fullname()
        reporter = Reporter(project_dir=os.curdir, sdist_dir=base_dir)
        try:
            report = reporter.make_report()
        finally:
            self.keep_temp = _saved_keep_temp
            if not self.keep_temp:
                distutils.dir_util.remove_tree(base_dir, dry_run=self.dry_run)
        _log.info("Wrote archive to: %s" % self.archive_files)
        reporter.show_report(report, self.archive_files, egg_dirs)


class pizza_prep(Command):
    """
    Prepare the project for pushing to PyPI.

    In particular, this updates the long_description file, which should
    be committed prior to pushing to PyPI.

    """
    description = ("prepare the project for pushing to PyPI")

    # Attributes and methods required by distutils.
    user_options = []
    def initialize_options(self): pass
    def finalize_options(self): pass

    def run(self):
        write_long_description(LONG_DESCRIPTION_PATH)


# The purpose of this function is to follow the guidance suggested here:
# http://packages.python.org/distribute/python3.html#note-on-compatibility-with-setuptools
#
# The guidance is for better compatibility when using setuptools (e.g. with
# earlier versions of Python 2) instead of Distribute, because of new
# keyword arguments to setup() that setuptools may not recognize.
def get_extra_args():
    """
    Return a dictionary of extra args to pass to setup().

    """
    extra = {}
    # TODO [template]: document that Distribute is necessary if using Python 3 and
    # possibly include this in the exception message.
    # Check the Python version instead of whether we're using Distribute or
    # setuptools because the former is less brittle.
    if sys.version_info >= (3, ):
        # Causes 2to3 to be run during the build step.
        extra['use_2to3'] = True

    return extra

CLASSIFIERS = (
    'Development Status :: 2 - Pre-Alpha',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: Implementation :: PyPy',
)

def main(sys_argv):
    """
    Call setup() with the correct arguments.

    """
    configure_logging()
    package_dir = os.path.join(os.path.dirname(__file__), PACKAGE_NAME)
    version = utils.scrape_version(package_dir)

    _log.info("running version: %s" % version)
    _log.info("using: version %r (%s) of %r" % (dist.__version__,
                                                dist_version, dist))

    long_description = get_long_description()
    extra_args = get_extra_args()
    # We do not include pizza_setup to prevent it from going into the
    # build/install.  However, this does not prevent it from going into the
    # source distribution, where it should go (which we do via the
    # MANIFEST.in file).  Also, we currently include the test subpackages.
    # For information on excluding test packages, see:
    # http://packages.python.org/distribute/setuptools.html#using-find-packages
    # TODO [template]: add a tox test to check that pizza_setup is not installed.
    packages = setuptools.find_packages(exclude=['pizza_setup',
                                                 'pizza_setup.*'])

    if extra_args:
        _log.info('including extra kwargs: %r' % extra_args)

    setup(name='Pizza',
          version=version,
          description='a model project for a Python command-line script',
          long_description=long_description,
          keywords='project template groome molt pystache mustache',
          author='Chris Jerdonek',
          author_email='chris.jerdonek@gmail.com',
          url='https://github.com/cjerdonek/groome-python-expected',
          packages=packages,
          classifiers=CLASSIFIERS,
          cmdclass = {pizza_prep.__name__: pizza_prep,
                      'register': register,
                      'sdist': sdist,
                      'upload': upload},
    #      install_requires=INSTALL_REQUIRES,
    #      package_data=package_data,
          entry_points = {
            'console_scripts': [
                'pizza=pizza.scripts.pizza.main:main',
            ],
          },
          **extra_args
    )

if __name__=='__main__':
    main(sys.argv)



"""
Tests of pizza.pizza.

"""

import unittest
import pizza.test.harness.general.loading as loading

# TODO [template]: use this only where it makes sense.
load_tests = loading.config_load_tests

class MainTestCase(unittest.TestCase):

    def test(self):
        self.assertEqual(self.test_config.temp_dir, "TODO")
        self.assertEqual(1, 1)



# encoding: utf-8

"""
Contains argument-parsing code and command-line documentation.

"""

import argparse
import sys

import pizza
import pizza.general.optionparser as _parsing

METAVAR_ARG_VALUE = 'VALUE'
METAVAR_INPUT_DIR = 'DIRECTORY'

# TODO [template]: rename to OPTION_*...
OPTION_HELP = _parsing.Option(('-h', '--help'))
FLAGS_LICENSE = _parsing.Option(('--license',))
FLAGS_MODE_TESTS = _parsing.Option(('-T', '--run-tests',))
FLAGS_SDIST_DIR = _parsing.Option(('--sdist-dir',))
OPTION_VERBOSE = _parsing.Option(('-v', '--verbose'))

# TODO [template]: populate with sample.json description and URL.
DESCRIPTION = """\
Make a pizza!
"""

EPILOG = "This is version %s of Pizza." % pizza.__version__

COPYRIGHT_LINE = "Copyright (C) 2011-2013 Chris Jerdonek. All rights reserved."

LICENSE_STRING = """\
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
* The names of the copyright holders may not be used to endorse or promote
  products derived from this software without specific prior written
  permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
"""

# This dict lets us group help strings together for easier editing.
HELP_STRINGS = {
    'args': """\
zero more input values.
""",
    FLAGS_MODE_TESTS: """\
discover and run project tests.  Tests include unit tests and doctests.
Running this command is for the most part equivalent to running unittest's
command-line discover command with an appropriate -t/--start-directory value.
Option values are passed along as is to the discover command.  For info on
the discovery options, consult the Python documentation or pass -h or --help
as an option to this value.
""",
    FLAGS_SDIST_DIR: """\
the path to the source distribution directory (aka sdist) if running
from a source checkout.  Otherwise, this option should be left out.
""",
    OPTION_HELP: """\
show this help message and exit.
""",
}

def _get_version_header():
    return "Molt %s" % pizza.__version__


def get_version_string():
    using_string = "Using: Python %s\n at %s" % (sys.version, sys.executable)
    s = "\n\n".join([_get_version_header(), using_string, COPYRIGHT_LINE])
    return s


def get_license_string():
    s = "\n\n".join([_get_version_header(), COPYRIGHT_LINE, LICENSE_STRING])
    return s


def preparse_args(sys_argv):
    """
    Parse command arguments without raising an exception (or exiting).

    This function allows one to have access to the command-line options
    before configuring logging (in particular before exception logging).

    Returns a Namespace object, or None if UsageError.

    """
    try:
        # Suppress the help option to prevent exiting.
        ns = parse_args(sys_argv, suppress_help_exit=True)
    except _parsing.UsageError:
        # Any usage error will occur again during the real parse.
        return None
    return ns


def parse_args(sys_argv, suppress_help_exit=False):
    """
    Parse arguments and return a Namespace object.

    Raises UsageError on command-line usage error.

    """
    parser = _create_parser(suppress_help_exit=suppress_help_exit)
    return parser.parse_args(sys_argv[1:])


def _create_parser(suppress_help_exit=False):
    """
    Return an ArgParser for the program.

    """
    parser = _parsing.ArgParser(description=DESCRIPTION,
                       epilog=EPILOG,
                       add_help=False,
                       # Preserves formatting of the description and epilog.
                       formatter_class=argparse.RawDescriptionHelpFormatter)

    def add_arg(obj, option, help=None, **kwargs):
        """
        Arguments:
          obj: a parser or group.
          option: an option string or tuple of one or more option strings.

        """
        if help is None:
            help = HELP_STRINGS[option]
        if isinstance(option, basestring):
            option = (option, )
        obj.add_argument(*option, help=help, **kwargs)

    # TODO [template]: incorporate the METAVAR names into the help messages,
    # as appropriate.
    # TODO [template]: fix the help message.
    add_arg(parser, 'args', metavar=METAVAR_ARG_VALUE, nargs='*')
    # This argument is the path to a source checkout or source distribution.
    # This lets one specify project resources not available in a package
    # build or install, when doing development testing.  Defaults to no
    # source directory.
    add_arg(parser, FLAGS_SDIST_DIR, metavar='DIRECTORY', dest='sdist_dir',
            action='store', default=None)
    add_arg(parser, OPTION_VERBOSE, dest='verbose', action='store_true',
            help='log verbosely.')

    # This group corresponds to the possible "modes" or "commands".
    # We do not use a subparsers for this because of issue #17050:
    # http://bugs.python.org/issue17050
    group = parser.add_mutually_exclusive_group()
    # run_tests is None if not provided, otherwise a list.
    add_arg(group, FLAGS_MODE_TESTS, dest='run_tests',
            nargs=argparse.REMAINDER)
    add_arg(group, FLAGS_LICENSE, dest='license_mode', action='store_true',
            help='print license info to stdout.')
    add_arg(group, ('-V', '--version'), dest='version_mode',
            action='store_true', help='print version info to stdout.')
    # We add help manually for more control.
    help_action = "store_true" if suppress_help_exit else "help"
    add_arg(group, OPTION_HELP, action=help_action)

    return parser



"""
Exposes the function that the run_test command-line option should call.

"""

import logging
import unittest

import pizza.test
import pizza.test.harness.general.loading as loading


log = logging.getLogger("pizza.test")

class TestConfig(object):

    def __init__(self, temp_dir):
        """
        Arguments:

          temp_dir: the sandbox directory that test cases can use for writing
            temporary files to the file system.

        """
        self.temp_dir = temp_dir


def run_tests(argv):
    log.info("run_tests argv: %r" % argv)
    # TODO [template]: pass the correct directory.
    config = TestConfig(temp_dir="TODO")
    loader = loading.TestLoader()
    loader.test_config = config
    TestPizza(argv=argv, testLoader=loader)


class TestPizza(unittest.TestProgram):

    # We need to override this method because of CPython issue #17052:
    # http://bugs.python.org/issue17052
    def _do_discovery(self, argv, Loader=None):
        if Loader is None:
            Loader = lambda: self.testLoader
        super(TestPizza, self)._do_discovery(argv, Loader=Loader)

    def parseArgs(self, argv):
        super(TestPizza, self).parseArgs(argv)



"""
Contains code specific to using this project from the command-line.

In particular, this package contains the console_script entries for the
entry_points argument to setup.py's setup() function.
"""



# encoding: utf-8
# Project auto-generated at: ...
# Copyright (C) 2012 Mustachioed Maven.

"""
A convenience script for running the main console script while developing.

"""

from pizza.scripts.pizza import main as main_module


def main():
    # The body of this function should be the same as the contents of the
    # __main__.py file in the subpackage containing main_module.
    main_module.main(from_source=True)

if __name__ == "__main__":
    main()
